@@ -68,7 +68,7 @@ NOEXPORT int main_unix(int argc, char* argv[]) {
     if(fd<0)
         fatal("Could not open /dev/null");
 #endif
-    main_initialize();
+    main_init();
     if(main_configure(argc>1 ? argv[1] : NULL, argc>2 ? argv[2] : NULL)) {
         close(fd);
         return 1;
@@ -108,6 +108,8 @@ NOEXPORT int main_unix(int argc, char* argv[]) {
         signal(SIGCHLD, SIG_IGN); /* ignore dead children */
         signal(SIGPIPE, SIG_IGN); /* ignore broken pipe */
 #endif
+        set_nonblock(0, 1); /* stdin */
+        set_nonblock(1, 1); /* stdout */
         client_main(alloc_client_session(&service_options, 0, 1));
     }
     return 0;
@@ -205,7 +207,7 @@ NOEXPORT void delete_pid(void) {
 
 /**************************************** options callbacks */
 
-void ui_new_config(void) {
+void ui_config_reloaded(void) {
     /* no action */
 }
 
